from django.core.urlresolvers import reverse
from django.utils.six.moves.urllib import parse
from djactasauth.backends import ActAsModelBackend


def act_as_login_url(auth, act_as, **query):
    username = ActAsModelBackend.sepchar.join([auth, act_as])
    return get_login_url(username=username, **query)


def get_login_url(**query):
    return reverse('login') + '?' + parse.urlencode(query)



class InitialValuesFromRequestGetFormMixin(object):

    query2initial = tuple()

    def __init__(self, *a, **kw):
        super(InitialValuesFromRequestGetFormMixin, self).__init__(*a, **kw)
        field_names = set(self.query2initial)
        for field_name in field_names:
            val_from_query = self.request.GET.get(field_name, None)
            if val_from_query:
                self.initial.setdefault(field_name, val_from_query)



import django
from django.contrib.auth.views import login
from django.contrib.auth.forms import AuthenticationForm
from djactasauth.forms import InitialValuesFromRequestGetFormMixin


def get_login_form(request, authentication_form=AuthenticationForm, **kwargs):
    base_form = authentication_form
    if django.VERSION[:2] < (1, 6):
        class CaptureRequestActAsAuthForm(base_form):
            def __init__(self, *a, **kw):
                if request.method == 'POST':
                    kw.setdefault('request', request)  # "backport" from 1.6
                super(CaptureRequestActAsAuthForm, self).__init__(*a, **kw)

        base_form = CaptureRequestActAsAuthForm

    if not issubclass(base_form, InitialValuesFromRequestGetFormMixin):
        class InitialFromQueryAuthForm(
                InitialValuesFromRequestGetFormMixin, base_form):

            @property
            def query2initial(self):
                return super(InitialFromQueryAuthForm, self).query2initial + \
                        ('username',)

        base_form = InitialFromQueryAuthForm
    else:
        # TODO: raise warning if it doesn't have username in query2initial
        pass

    return base_form


def act_as_login_view(request, **kwargs):
    kwargs['authentication_form'] = get_login_form(request, **kwargs)
    return login(request, **kwargs)



# -*- coding: utf-8 -*-
from django.contrib.auth.backends import ModelBackend
from django.contrib.auth import get_user_model


class FilteredModelBackend(ModelBackend):
    def get_user(self, user_id):
        user = super(FilteredModelBackend, self).get_user(user_id)
        return self.filter_user(user)

    def authenticate(self, username=None, password=None, **kwargs):
        user = super(FilteredModelBackend, self).authenticate(
                username=username, password=password, **kwargs)
        return self.filter_user(user)

    def filter_user(self, user):
        if not user:
            return user
        filters = getattr(self, 'filter_kwargs', None)
        if filters:
            qs = type(user)._default_manager.filter(
                pk=user.pk).filter(**filters)
            if not qs.exists():
                return None
        return user


class ActAsModelBackend(FilteredModelBackend):

    sepchar = '/'

    @classmethod
    def is_act_as_username(cls, username):
        if not username:
            return False
        return cls.sepchar in username

    def authenticate(self, username=None, password=None, **kwargs):
        if self.is_act_as_username(username):
            auth_username, act_as_username = username.split(self.sepchar)
        else:
            auth_username = act_as_username = username

        auth_user = super(ActAsModelBackend, self).authenticate(
                username=auth_username, password=password, **kwargs)
        if not auth_user:
            return auth_user
        if auth_username != act_as_username:
            UserModel = get_user_model()
            try:
                user = UserModel._default_manager.get_by_natural_key(
                    act_as_username)
            except UserModel.DoesNotExist:
                return None
            if not self.can_act_as(auth_user=auth_user, user=user):
                return None
        else:
            user = auth_user
        return user

    def can_act_as(self, auth_user, user):
        return False


class OnlySuperuserCanActAsModelBackend(ActAsModelBackend):
    def can_act_as(self, auth_user, user):
        return auth_user.is_superuser and not user.is_superuser



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import sys

import djactasauth

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

version = djactasauth.__version__

if sys.argv[-1] == 'publish':
    os.system('make release')
    sys.exit()

setup(
    name='djactasauth',
    version=version,
    author='Paessler AG',
    url='https://github.com/PaesslerAG/django-act-as-auth',
    packages=[
        'djactasauth',
    ],
    include_package_data=True,
    install_requires=[
        'Django>=1.5',
    ],
    license="BSD",
    zip_safe=False,
    keywords='django, authentication, act as, impersonation',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],
)
